@@ -41,17 +41,18 @@ class QuadrilateralMixin(object):
 
 def make_attribs(additional=None):
     dxfattribs = {
-        'handle': DXFAttr(5, None),
-        'layer': DXFAttr(8, None),  # layername as string, default is '0'
-        'linetype': DXFAttr(6, None),  # linetype as string, special names BYLAYER/BYBLOCK, default is BYLAYER
-        'color': DXFAttr(62, None),  # dxf color index, 0 .. BYBLOCK, 256 .. BYLAYER, default is 256
-        'paperspace': DXFAttr(67, None),  # 0 .. modelspace, 1 .. paperspace, default is 0
-        'extrusion': DXFAttr(210, 'Point3D'),  # never used !?
+        'handle': DXFAttr(5),
+        'layer': DXFAttr(8, default='0'),  # layername as string
+        'linetype': DXFAttr(6, default='BYLAYER'),  # linetype as string, special names BYLAYER/BYBLOCK
+        'color': DXFAttr(62, default=256),  # dxf color index, 0 .. BYBLOCK, 256 .. BYLAYER
+        'paperspace': DXFAttr(67, default=0),  # 0 .. modelspace, 1 .. paperspace
+        'extrusion': DXFAttr(210, xtype='Point3D', default=(0.0, 0.0, 1.0)),  # Z-axis of OCS (Object-Coordinate-System)
     }
     if additional is not None:
         dxfattribs.update(additional)
     return DXFAttributes(DefSubclass(None, dxfattribs))
 
+
 class GraphicEntity(GenericWrapper):
     """ Default graphic entity wrapper, allows access to following dxf attributes:
 
@@ -67,15 +68,13 @@ class GraphicEntity(GenericWrapper):
     DXFATTRIBS = make_attribs()
 
     def set_layout(self, layout):
+        # required because for DXF12 block entities has no pointer to the associated block layout (owner_id)
         self.layout = layout
 
     @property
     def dxffactory(self):
-        return self.layout._dxffactory
+        return self.drawing.dxffactory
 
-    @property
-    def drawing(self):
-        return self.layout._dxffactory.drawing
 
 _LINE_TPL = """  0
 LINE
@@ -101,8 +100,8 @@ LINE
 class Line(GraphicEntity, ColorMixin):
     TEMPLATE = ClassifiedTags.from_text(_LINE_TPL)
     DXFATTRIBS = make_attribs({
-        'start': DXFAttr(10, 'Point2D/3D'),
-        'end': DXFAttr(11, 'Point2D/3D'),
+        'start': DXFAttr(10, xtype='Point2D/3D'),
+        'end': DXFAttr(11, xtype='Point2D/3D'),
     })
 
 _POINT_TPL = """  0
@@ -123,7 +122,7 @@ POINT
 class Point(GraphicEntity, ColorMixin):
     TEMPLATE = ClassifiedTags.from_text(_POINT_TPL)
     DXFATTRIBS = make_attribs({
-        'point': DXFAttr(10, 'Point2D/3D'),
+        'location': DXFAttr(10, xtype='Point2D/3D'),
     })
 
 _CIRCLE_TPL = """  0
@@ -146,8 +145,8 @@ CIRCLE
 class Circle(GraphicEntity, ColorMixin):
     TEMPLATE = ClassifiedTags.from_text(_CIRCLE_TPL)
     DXFATTRIBS = make_attribs({
-        'center': DXFAttr(10, 'Point2D/3D'),
-        'radius': DXFAttr(40, None),
+        'center': DXFAttr(10, xtype='Point2D/3D'),
+        'radius': DXFAttr(40),
     })
 
 _ARC_TPL = """  0
@@ -174,10 +173,10 @@ ARC
 class Arc(GraphicEntity, ColorMixin):
     TEMPLATE = ClassifiedTags.from_text(_ARC_TPL)
     DXFATTRIBS = make_attribs({
-        'center': DXFAttr(10, 'Point2D/3D'),
-        'radius': DXFAttr(40, None),
-        'start_angle': DXFAttr(50, None),
-        'end_angle': DXFAttr(51, None),
+        'center': DXFAttr(10, xtype='Point2D/3D'),
+        'radius': DXFAttr(40),
+        'start_angle': DXFAttr(50),
+        'end_angle': DXFAttr(51),
     })
 
 _TRACE_TPL = """  0
@@ -216,10 +215,10 @@ TRACE
 class Trace(GraphicEntity, ColorMixin, QuadrilateralMixin):
     TEMPLATE = ClassifiedTags.from_text(_TRACE_TPL)
     DXFATTRIBS = make_attribs({
-        'vtx0': DXFAttr(10, 'Point2D/3D'),
-        'vtx1': DXFAttr(11, 'Point2D/3D'),
-        'vtx2': DXFAttr(12, 'Point2D/3D'),
-        'vtx3': DXFAttr(13, 'Point2D/3D'),
+        'vtx0': DXFAttr(10, xtype='Point2D/3D'),
+        'vtx1': DXFAttr(11, xtype='Point2D/3D'),
+        'vtx2': DXFAttr(12, xtype='Point2D/3D'),
+        'vtx3': DXFAttr(13, xtype='Point2D/3D'),
     })
 
 
@@ -230,11 +229,11 @@ class Solid(Trace):
 class Face(Trace):
     TEMPLATE = ClassifiedTags.from_text(_TRACE_TPL.replace('TRACE', '3DFACE'))
     DXFATTRIBS = make_attribs({
-        'vtx0': DXFAttr(10, 'Point2D/3D'),
-        'vtx1': DXFAttr(11, 'Point2D/3D'),
-        'vtx2': DXFAttr(12, 'Point2D/3D'),
-        'vtx3': DXFAttr(13, 'Point2D/3D'),
-        'invisible_edge': DXFAttr(70, None),
+        'vtx0': DXFAttr(10, xtype='Point3D'),
+        'vtx1': DXFAttr(11, xtype='Point3D'),
+        'vtx2': DXFAttr(12, xtype='Point3D'),
+        'vtx3': DXFAttr(13, xtype='Point3D'),
+        'invisible_edge': DXFAttr(70, default=0),
     })
 
 _TEXT_TPL = """  0
@@ -279,18 +278,19 @@ STANDARD
 class Text(GraphicEntity, ColorMixin):
     TEMPLATE = ClassifiedTags.from_text(_TEXT_TPL)
     DXFATTRIBS = make_attribs({
-        'insert': DXFAttr(10, 'Point2D/3D'),
-        'height': DXFAttr(40,  None),
-        'text': DXFAttr(1,  None),
-        'rotation': DXFAttr(50, None),  # in degrees (circle = 360deg)
-        'oblique': DXFAttr(51, None),  # in degrees, vertical = 0deg
-        'style': DXFAttr(7, None),  # text style
-        'width': DXFAttr(41, None),  # width FACTOR!
-        'text_generation_flag': DXFAttr(71, None),  # 2 = backward (mirr-x), 4 = upside down (mirr-y)
-        'halign': DXFAttr(72, None),  # horizontal justification
-        'valign': DXFAttr(73,  None),  # vertical justification
-        'align_point': DXFAttr(11, 'Point2D/3D'),
+        'insert': DXFAttr(10, xtype='Point2D/3D'),
+        'height': DXFAttr(40),
+        'text': DXFAttr(1),
+        'rotation': DXFAttr(50, default=0.0),  # in degrees (circle = 360deg)
+        'oblique': DXFAttr(51, default=0.0),  # in degrees, vertical = 0deg
+        'style': DXFAttr(7, default='STANDARD'),  # text style
+        'width': DXFAttr(41, default=1.0),  # width FACTOR!
+        'text_generation_flag': DXFAttr(71, default=0),  # 2 = backward (mirr-x), 4 = upside down (mirr-y)
+        'halign': DXFAttr(72, default=0),  # horizontal justification
+        'valign': DXFAttr(73,  default=0),  # vertical justification
+        'align_point': DXFAttr(11, xtype='Point2D/3D'),
     })
+
     def set_pos(self, p1, p2=None, align=None):
         if align is None:
             align = self.get_align()
@@ -355,17 +355,17 @@ BLOCKNAME
 class Block(GraphicEntity):
     TEMPLATE = ClassifiedTags.from_text(_BLOCK_TPL)
     DXFATTRIBS = make_attribs({
-        'name': DXFAttr(2, None),
-        'name2': DXFAttr(3, None),
-        'flags': DXFAttr(70, None),
-        'base_point': DXFAttr(10, 'Point2D/3D'),
-        'xref_path': DXFAttr(1, None),
+        'name': DXFAttr(2),
+        'name2': DXFAttr(3),
+        'flags': DXFAttr(70),
+        'base_point': DXFAttr(10, xtype='Point2D/3D'),
+        'xref_path': DXFAttr(1),
     })
 
 
 class EndBlk(GraphicEntity):
     TEMPLATE = ClassifiedTags.from_text("  0\nENDBLK\n  5\n0\n")
-    DXFATTRIBS = DXFAttributes(DefSubclass(None, {'handle': DXFAttr(5, None)}))
+    DXFATTRIBS = DXFAttributes(DefSubclass(None, {'handle': DXFAttr(5)}))
 
 _INSERT_TPL = """  0
 INSERT
@@ -398,46 +398,73 @@ BLOCKNAME
 class Insert(GraphicEntity):
     TEMPLATE = ClassifiedTags.from_text(_INSERT_TPL)
     DXFATTRIBS = make_attribs({
-        'attribs_follow': DXFAttr(66, None),
-        'name': DXFAttr(2, None),
-        'insert': DXFAttr(10, 'Point2D/3D'),
-        'xscale': DXFAttr(41, None),
-        'yscale': DXFAttr(42, None),
-        'zscale': DXFAttr(43, None),
-        'rotation': DXFAttr(50, None),
-        'column_count': DXFAttr(70, None),
-        'row_count': DXFAttr(71, None),
-        'column_spacing': DXFAttr(44, None),
-        'row_spacing': DXFAttr(45, None),
+        'attribs_follow': DXFAttr(66, default=0),
+        'name': DXFAttr(2),
+        'insert': DXFAttr(10, xtype='Point2D/3D'),
+        'xscale': DXFAttr(41, default=1.0),
+        'yscale': DXFAttr(42, default=1.0),
+        'zscale': DXFAttr(43, default=1.0),
+        'rotation': DXFAttr(50, default=0.0),
+        'column_count': DXFAttr(70, default=1),
+        'row_count': DXFAttr(71, default=1),
+        'column_spacing': DXFAttr(44, default=0.0),
+        'row_spacing': DXFAttr(45, default=0.0),
     })
 
     def __iter__(self):
-        def get_entity(index):
-            try:
-                return self.layout.get_entity_at_index(index)
-            except IndexError:
-                raise DXFStructureError('expected following ATTRIB or SEQEND, reached end of layout instead.')
-
         if self.dxf.attribs_follow == 0:
             return
-        index = self.layout.get_index_of_entity(self) + 1
+        cursor = self.layout.get_cursor(self)
         while True:
-            entity = get_entity(index)
+            entity = cursor.next_entity()
             dxftype = entity.dxftype()
             if dxftype == 'ATTRIB':
                 yield entity
-                index += 1
             elif dxftype == 'SEQEND':
                 return
             else:
                 raise DXFStructureError('expected following ATTRIB or SEQEND, got instead %s.' % dxftype)
 
+    def place(self, insert=None, scale=None, rotation=None):
+        if insert is not None:
+            self.dxf.insert = insert
+        if scale is not None:
+            if len(scale) != 3:
+                raise ValueError("Parameter scale has to be a 3-tuple.")
+            x, y, z = scale
+            self.dxf.xscale = x
+            self.dxf.yscale = y
+            self.dxf.zscale = z
+        if rotation is not None:
+            self.dxf.rotation = rotation
+        return self
+
+    def grid(self, size=(1, 1), spacing=(1, 1)):
+        if len(size) != 2:
+            raise ValueError("Parameter size has to be a (row_count, column_count)-tuple.")
+        if len(spacing) != 2:
+            raise ValueError("Parameter spacing has to be a (row_spacing, column_spacing)-tuple.")
+        self.dxf.row_count = size[0]
+        self.dxf.column_count = size[1]
+        self.dxf.row_spacing = spacing[0]
+        self.dxf.column_spacing = spacing[1]
+        return self
+
     def get_attrib(self, tag):
         for attrib in self:
             if tag == attrib.dxf.tag:
                 return attrib
         return None
 
+    def get_attrib_text(self, tag, default=None):
+        attrib = self.get_attrib(tag)
+        if attrib is None:
+            return default
+        return attrib.dxf.text
+
+    def has_attrib(self, tag):
+        return self.get_attrib(tag) is not None
+
     def add_attrib(self, tag, text, insert, dxfattribs=None):
         if dxfattribs is None:
             dxfattribs = {}
@@ -448,35 +475,52 @@ class Insert(GraphicEntity):
         self._append_attrib_entity(attrib_entity)
 
     def _append_attrib_entity(self, entity):
-        def find_seqend(pos):
+        def find_seqend(cursor):
             while True:
                 try:
-                    entity = self.layout.get_entity_at_index(pos)
+                    entity = cursor.next_entity()
                 except IndexError:
                     return -1
                 dxftype = entity.dxftype()
                 if dxftype == 'ATTRIB':
-                    pos += 1
+                    pass
                 elif dxftype == 'SEQEND':
-                    return pos
+                    return cursor.pos
                 else:
                     return -1
 
         entities = [entity]
-        position = self.layout.get_index_of_entity(self) + 1
-        seqend_position = find_seqend(position)
+        cursor = self.layout.get_cursor(self)
+        insert_position = cursor.pos
+        seqend_position = find_seqend(cursor)
         if seqend_position < 0:
             entities.append(self.layout.build_entity('SEQEND', {}))
-            seqend_position = position
+            seqend_position = insert_position + 1
         self.dxf.attribs_follow = 1
         self.layout.insert_entities(seqend_position, entities)
 
+    def destroy(self):
+        if not hasattr(self, 'layout') or self.layout is None:
+            return  # created without layout attribute
+        if self.dxf.attribs_follow == 0:  # no following ATTRIB entities
+            return
+
+        following_entities = []
+        cursor = self.layout.get_cursor(self)
+        while True:
+            entity = cursor.next_entity()
+            following_entities.append(entity)  # collect entities, because deleting irritates layout cursor
+            if entity.dxftype() == 'SEQEND':
+                break
+        for entity in following_entities:
+            self.layout.delete_entity(entity)
+
 
 class SeqEnd(GraphicEntity):
     TEMPLATE = ClassifiedTags.from_text("  0\nSEQEND\n  5\n0\n")
     DXFATTRIBS = DXFAttributes(DefSubclass(None, {
-        'handle': DXFAttr(5, None),
-        'paperspace': DXFAttr(67, None),
+        'handle': DXFAttr(5),
+        'paperspace': DXFAttr(67, default=0),
     }))
 
 _ATTDEF_TPL = """  0
@@ -526,24 +570,24 @@ STANDARD
 """
 
 
-class Attdef(GraphicEntity):
+class Attdef(Text):
     TEMPLATE = ClassifiedTags.from_text(_ATTDEF_TPL)
     DXFATTRIBS = make_attribs({
-        'insert': DXFAttr(10, 'Point2D/3D'),
-        'height': DXFAttr(40, None),
-        'text': DXFAttr(1, None),
-        'prompt': DXFAttr(3, None),
-        'tag': DXFAttr(2, None),
-        'flags': DXFAttr(70, None),
-        'field_length': DXFAttr(73, None),
-        'rotation': DXFAttr(50, None),
-        'oblique': DXFAttr(51, None),
-        'width': DXFAttr(41, None),  # width factor
-        'style': DXFAttr(7, None),
-        'text_generation_flag': DXFAttr(71, None),  # 2 = backward (mirr-x), 4 = upside down (mirr-y)
-        'halign': DXFAttr(72, None),  # horizontal justification
-        'valign': DXFAttr(74, None),  # vertical justification
-        'align_point': DXFAttr(11, 'Point2D/3D'),
+        'insert': DXFAttr(10, xtype='Point2D/3D'),
+        'height': DXFAttr(40),
+        'text': DXFAttr(1),
+        'prompt': DXFAttr(3),
+        'tag': DXFAttr(2),
+        'flags': DXFAttr(70),
+        'field_length': DXFAttr(73, default=0),
+        'rotation': DXFAttr(50, default=0.0),
+        'oblique': DXFAttr(51, default=0.0),
+        'width': DXFAttr(41, default=1.0),  # width factor
+        'style': DXFAttr(7, default='STANDARD'),
+        'text_generation_flag': DXFAttr(71, default=0),  # 2 = backward (mirr-x), 4 = upside down (mirr-y)
+        'halign': DXFAttr(72, default=0),  # horizontal justification
+        'valign': DXFAttr(74, default=0),  # vertical justification
+        'align_point': DXFAttr(11, xtype='Point2D/3D'),
     })
 
 _ATTRIB_TPL = """  0
@@ -591,23 +635,23 @@ STANDARD
 """
 
 
-class Attrib(GraphicEntity):
+class Attrib(Text):
     TEMPLATE = ClassifiedTags.from_text(_ATTRIB_TPL)
     DXFATTRIBS = make_attribs({
-        'insert': DXFAttr(10, 'Point2D/3D'),
-        'height': DXFAttr(40, None),
-        'text': DXFAttr(1, None),
-        'tag': DXFAttr(2, None),
-        'flags': DXFAttr(70, None),
-        'field_length': DXFAttr(73, None),
-        'rotation': DXFAttr(50, None),
-        'oblique': DXFAttr(51, None),
-        'width': DXFAttr(41, None),  # width factor
-        'style': DXFAttr(7, None),
-        'text_generation_flag': DXFAttr(71, None),  # 2 = backward (mirr-x), 4 = upside down (mirr-y)
-        'halign': DXFAttr(72, None),  # horizontal justification
-        'valign': DXFAttr(74, None),  # vertical justification
-        'align_point': DXFAttr(11, 'Point2D/3D'),
+        'insert': DXFAttr(10, xtype='Point2D/3D'),
+        'height': DXFAttr(40),
+        'text': DXFAttr(1),
+        'tag': DXFAttr(2),
+        'flags': DXFAttr(70),
+        'field_length': DXFAttr(73, default=0),
+        'rotation': DXFAttr(50, default=0.0),
+        'oblique': DXFAttr(51, default=0.0),
+        'width': DXFAttr(41, default=1.0),  # width factor
+        'style': DXFAttr(7, default='STANDARD'),
+        'text_generation_flag': DXFAttr(71, default=0),  # 2 = backward (mirr-x), 4 = upside down (mirr-y)
+        'halign': DXFAttr(72, default=0),  # horizontal justification
+        'valign': DXFAttr(74, default=0),  # vertical justification
+        'align_point': DXFAttr(11, xtype='Point2D/3D'),
     })
 
 _POLYLINE_TPL = """  0
@@ -632,15 +676,15 @@ POLYLINE
 class Polyline(GraphicEntity, ColorMixin):
     TEMPLATE = ClassifiedTags.from_text(_POLYLINE_TPL)
     DXFATTRIBS = make_attribs({
-        'elevation': DXFAttr(10, 'Point2D/3D'),
-        'flags': DXFAttr(70, None),
-        'default_start_width': DXFAttr(40, None),
-        'default_end_width': DXFAttr(41, None),
-        'm_count': DXFAttr(71, None),
-        'n_count': DXFAttr(72, None),
-        'm_smooth_density': DXFAttr(73, None),
-        'n_smooth_density': DXFAttr(74, None),
-        'smooth_type': DXFAttr(75, None),
+        'elevation': DXFAttr(10, xtype='Point2D/3D'),
+        'flags': DXFAttr(70, default=0),
+        'default_start_width': DXFAttr(40, default=0.0),
+        'default_end_width': DXFAttr(41, default=0.0),
+        'm_count': DXFAttr(71, default=0),
+        'n_count': DXFAttr(72, default=0),
+        'm_smooth_density': DXFAttr(73, default=0),
+        'n_smooth_density': DXFAttr(74, default=0),
+        'smooth_type': DXFAttr(75, default=0),
     })
 
     def get_vertex_flags(self):
@@ -674,12 +718,11 @@ class Polyline(GraphicEntity, ColorMixin):
 
     def __iter__(self):
         """ Iterate over all vertices. """
-        index = self.layout.get_index_of_entity(self) + 1
-        entity = self.layout.get_entity_at_index(index)
+        cursor = self.layout.get_cursor(self)
+        entity = cursor.next_entity()
         while entity.dxftype() != 'SEQEND':
             yield entity
-            index += 1
-            entity = self.layout.get_entity_at_index(index)
+            entity = cursor.next_entity()
 
     def __getitem__(self, pos):
         return list(self)[pos]
@@ -728,13 +771,12 @@ class Polyline(GraphicEntity, ColorMixin):
             raise IndexError(repr((pos, count)))
 
     def _get_index_range(self):
-        first_vertex_index = self.layout.get_index_of_entity(self) + 1
-        last_vertex_index = first_vertex_index
+        cursor = self.layout.get_cursor(self)
+        first_vertex_index = cursor.pos + 1
         while True:
-            entity = self.layout.get_entity_at_index(last_vertex_index)
+            entity = cursor.next_entity()
             if entity.dxftype() == 'SEQEND':
-                return first_vertex_index, last_vertex_index - 1
-            last_vertex_index += 1
+                return first_vertex_index, cursor.pos - 1
 
     def cast(self):
         mode = self.get_mode()
@@ -747,8 +789,22 @@ class Polyline(GraphicEntity, ColorMixin):
 
     def _get_vertex_at_trusted_position(self, pos):
         # does no index check - for meshes and faces
-        index = self.layout.get_index_of_entity(self) + 1 + pos
-        return self.layout.get_entity_at_index(index)
+        cursor = self.layout.get_cursor(self)
+        cursor.skip(pos + 1)
+        return cursor.entity()
+
+    def destroy(self):
+        if not hasattr(self, 'layout') or self.layout is None:
+            return  # created without layout attribute
+        following_entities = []
+        cursor = self.layout.get_cursor(self)
+        while True:
+            entity = cursor.next_entity()
+            following_entities.append(entity)  # collect entities, because deleting irritates layout cursor
+            if entity.dxftype() == 'SEQEND':
+                break
+        for entity in following_entities:
+            self.layout.delete_entity(entity)
 
 
 class Polyface(Polyline, PolyfaceMixin):
@@ -792,16 +848,16 @@ VERTEX
 class Vertex(GraphicEntity, ColorMixin, QuadrilateralMixin):
     TEMPLATE = ClassifiedTags.from_text(_VERTEX_TPL)
     DXFATTRIBS = make_attribs({
-        'location': DXFAttr(10, 'Point2D/3D'),
-        'start_width': DXFAttr(40, None),
-        'end_width': DXFAttr(41, None),
-        'bulge': DXFAttr(42, None),
-        'flags': DXFAttr(70, None),
-        'tangent': DXFAttr(50, None),
-        'vtx0': DXFAttr(71, None),
-        'vtx1': DXFAttr(72, None),
-        'vtx2': DXFAttr(73, None),
-        'vtx3': DXFAttr(74, None),
+        'location': DXFAttr(10, xtype='Point2D/3D'),
+        'start_width': DXFAttr(40, default=0.0),
+        'end_width': DXFAttr(41, default=0.0),
+        'bulge': DXFAttr(42, default=0),
+        'flags': DXFAttr(70, default=0),
+        'tangent': DXFAttr(50),
+        'vtx0': DXFAttr(71),
+        'vtx1': DXFAttr(72),
+        'vtx2': DXFAttr(73),
+        'vtx3': DXFAttr(74),
     })
     
 _VPORT_TPL = """  0
@@ -840,14 +896,11 @@ MVIEW
 class Viewport(GraphicEntity):
     TEMPLATE = ClassifiedTags.from_text(_VPORT_TPL)
     DXFATTRIBS = make_attribs({
-        'center': DXFAttr(10, 'Point2D/3D'),
-        # center point of entity in paper space coordinates)
-        'width': DXFAttr(40, None),
-        # width in paper space units
-        'height': DXFAttr(41, None),
-        # height in paper space units
-        'status': DXFAttr(68, None),
-        'id': DXFAttr(69, None),
+        'center': DXFAttr(10, xtype='Point2D/3D'),  # center point of entity in paper space coordinates)
+        'width': DXFAttr(40),  # width in paper space units
+        'height': DXFAttr(41),  # height in paper space units
+        'status': DXFAttr(68),
+        'id': DXFAttr(69),
     })
 
 _DIMENSION_TPL = """  0
@@ -914,18 +967,57 @@ DIMSTYLE
 class Dimension(GraphicEntity):
     TEMPLATE = ClassifiedTags.from_text(_DIMENSION_TPL)
     DXFATTRIBS = make_attribs({
-        'geometry': DXFAttr(2, None),
-        # name of pseudo-Block containing the current dimension  entity geometry
-        'dimstyle': DXFAttr(3, None),
-        'defpoint1': DXFAttr(10, 'Point2D/3D'),
-        'midpoint': DXFAttr(11, 'Point2D/3D'),
-        'translationvector': DXFAttr(12, 'Point3D'),
-        'dimtype': DXFAttr(70, None),
-        'usertext': DXFAttr(1, None),
-        'defpoint2': DXFAttr(13, 'Point2D/3D'),
-        'defpoint3': DXFAttr(14, 'Point2D/3D'),
-        'defpoint4': DXFAttr(15, 'Point2D/3D'),
-        'defpoint5': DXFAttr(16, 'Point2D/3D'),
-        'leaderlength': DXFAttr(40, None),
-        'angle': DXFAttr(50, None),
+        'geometry': DXFAttr(2),  # name of pseudo-Block containing the current dimension  entity geometry
+        'dimstyle': DXFAttr(3),
+        'defpoint1': DXFAttr(10, xtype='Point2D/3D'),
+        'midpoint': DXFAttr(11, xtype='Point2D/3D'),
+        'translation_vector': DXFAttr(12, 'Point3D'),
+        'dimtype': DXFAttr(70),
+        'user_text': DXFAttr(1),
+        'defpoint2': DXFAttr(13, xtype='Point2D/3D'),
+        'defpoint3': DXFAttr(14, xtype='Point2D/3D'),
+        'defpoint4': DXFAttr(15, xtype='Point2D/3D'),
+        'defpoint5': DXFAttr(16, xtype='Point2D/3D'),
+        'leader_length': DXFAttr(40),
+        'angle': DXFAttr(50),
+        'horizontal_direction': DXFAttr(51),
+        'oblique_angle': DXFAttr(52),
+        'rotation_angle': DXFAttr(53),
+    })
+
+_SHAPE_TPL = """  0
+SHAPE
+  5
+0
+  8
+0
+ 10
+0.0
+ 20
+0.0
+ 30
+0.0
+ 40
+1.0
+  2
+NAME
+ 50
+0.0
+ 41
+1.0
+ 51
+0.0
+"""
+
+
+# SHAPE is not tested with real world DXF drawings!
+class Shape(GraphicEntity, ColorMixin):
+    TEMPLATE = ClassifiedTags.from_text(_SHAPE_TPL)
+    DXFATTRIBS = make_attribs({
+        'insert': DXFAttr(10, xtype='Point2D/3D'),
+        'size': DXFAttr(40),
+        'name': DXFAttr(2),
+        'rotation': DXFAttr(50, default=0.0),
+        'xscale': DXFAttr(41, default=1.0),
+        'oblique': DXFAttr(51, default=0.0),
     })
